% PUF-Hamming-45-Security
% SC Intra-PUF vs SC Inter-PUF
%-------------------------------------------------------------------------
% File name   : function_cal_far_frr_SC_SC.m
% Version     : Version 1.0
% Created     : Hyunho Kang
% Last update : June/10/2011
% 
% Copyright (C) 2011 AIST
% 
% By using this code, you agree to the following terms and conditions.
% 
% This code is copyrighted by AIST ("us").
% 
% Permission is hereby granted to copy, reproduce, redistribute or
% otherwise use this code as long as: there is no monetary profit gained
% specifically from the use or reproduction of this code, it is not sold,
% rented, traded or otherwise marketed, and this copyright notice is
% included prominently in any copy made.
% 
% We shall not be liable for any damages, including without limitation
% direct, indirect, incidental, special or consequential damages arising
% from the use of this code.
% 
% When you publish any results arising from the use of this code, we will
% appreciate it if you can cite our webpage
% (http://staff.aist.go.jp/akashi.satoh/SASEBO/en/index.html).
%-------------------------------------------------------------------------
clear
clc

%% SC Intra-PUF (the Same Challenge)
for dn = 1:45
    
    if dn == 1
        a = load(['puf_chip_intra',int2str(1)]);
        honin1 = a.(['puf_chip_intra',in2str(1]);
        honin = honin1';
        
    else
        a = load(['puf_chip_intra',int2str(dn)]);
        honin1 = a.(['puf_chip_intra',in2str(dn]);
        
        honin2 = honin1';
        honin = [honin;honin2]; 
    end
       
end

%% SC Inter-PUF (the Same Challenge)


load sc_inter_a_pufs_45
tanin = sc_inter_a_pufs_45';



%% d-prime
cs_hme = mean(honin); %honin mean value
 cs_tme = mean(tanin); %tanin mean value
 
 cha_ue = cs_hme - cs_tme;  %difference two mean value
 abs_ue = abs(cha_ue);  %%%
 
 cs_hva = var(honin);
 cs_tva = var(tanin);
 
 add_cs_tva = cs_hva +cs_tva;
 
 sq_ue = sqrt(add_cs_tva/2); %%%
 
 dprime = abs_ue/sq_ue   %decidability

 
%%
% parameter for plotting two distributions
cs_max = max([tanin ; honin]);
cs_min = min([tanin ; honin]);
cs_x = [cs_min:cs_max];


h=histc(honin,cs_x);
h_s = sum(h);
h_sr = h/h_s;    % honnin   y axis
 
h0=histc(tanin,cs_x);
h0_s = sum(h0);
h0_sr = h0/h0_s;    % tanin   y axis


% parameter for making FAR, FRR
ma_honin = max(honin);  % honin: LEFT
num_honin = length(honin);

mi_tanin = min(tanin);  % tanin: RIGHT
num_tanin = length(tanin);

% calculation of the step
x = [mi_tanin:ma_honin];
num = length (x);


% calculation of FAR and FRR

for i=1:num
    fr=0;
    fa=0;
    for j=1:num_honin
        if honin(j)>=x(i)
            fr=fr+1;
        end
    end
    for k=1:num_tanin
        if tanin(k)<x(i)
            fa=fa+1;
        end
    end
    FRR(i)=100*fr/num_honin;
    FAR(i)=100*fa/num_tanin;
end 

% find the eer
if num == 0
    eer = 0;
     im = 1;
    FAR(im) = 0;
    FRR(im) = 0;
else
    [vl,im]=min(abs(FAR-FRR));
    eer =  (FAR(im)+FRR(im))/2;
    ori_far = FAR(im);
end


FAR45 = FAR;
FRR45 = FRR;
save FAR45 FAR45
save FRR45 FRR45





figure
plot(cs_x, h_sr, '-b',cs_x, h0_sr, '-r','LineWidth',2);   
xlim([0 128])
legend('SC Intra-PUF','SC Inter-PUF','FontSize',18);
st2=sprintf('EER:  %.2f, FAR:  %.2f, FRR:  %.2f, dprime:  %.2f',eer,FAR45(im),FRR45(im), dprime);
title(st2,'FontSize',12);
xlabel('Hamming distance','FontSize',18);
ylabel ('Count rates','FontSize',18);
% print -dmeta plot_honin_tanin_SC_SC.emf

